@@ -42,6 +42,10 @@ class OpenApiSerializerFieldExtension(OpenApiGeneratorExtension['OpenApiSerializ
     """
     _registry: List['OpenApiSerializerFieldExtension'] = []
 
+    def get_name(self) -> Optional[str]:
+        """ return str for breaking out field schema into separate component """
+        return None
+
     @abstractmethod
     def map_serializer_field(self, auto_schema, direction):
         pass  # pragma: no cover
@@ -63,3 +67,14 @@ class OpenApiViewExtension(OpenApiGeneratorExtension['OpenApiViewExtension']):
     @abstractmethod
     def view_replacement(self):
         pass  # pragma: no cover
+
+
+class OpenApiFilterExtension(OpenApiGeneratorExtension['OpenApiFilterExtension']):
+    """
+
+    """
+    _registry: List['OpenApiFilterExtension'] = []
+
+    @abstractmethod
+    def get_schema_operation_parameters(self, auto_schema, *args, **kwargs):
+        pass  # pragma: no cover
