@@ -439,11 +439,8 @@ class AutoSchema(ViewInspector):
             )
             return build_basic_type(OpenApiTypes.STR)
 
-    def _map_serializer_field(self, field, direction, collect_meta=True):
-        if collect_meta:
-            meta = self._get_serializer_field_meta(field)
-        else:
-            meta = {}
+    def _map_serializer_field(self, field, direction):
+        meta = self._get_serializer_field_meta(field)
 
         if has_override(field, 'field'):
             override = get_override(field, 'field')
@@ -454,7 +451,7 @@ class AutoSchema(ViewInspector):
             elif isinstance(override, dict):
                 schema = override
             else:
-                schema = self._map_serializer_field(force_instance(override), direction, False)
+                schema = self._map_serializer_field(force_instance(override), direction)
 
             field_component_name = get_override(field, 'field_component_name')
             if field_component_name:
@@ -472,7 +469,17 @@ class AutoSchema(ViewInspector):
         serializer_field_extension = OpenApiSerializerFieldExtension.get_match(field)
         if serializer_field_extension:
             schema = serializer_field_extension.map_serializer_field(self, direction)
-            return append_meta(schema, meta)
+            if serializer_field_extension.get_name():
+                component = ResolvedComponent(
+                    name=serializer_field_extension.get_name(),
+                    type=ResolvedComponent.SCHEMA,
+                    schema=schema,
+                    object=field,
+                )
+                self.registry.register_on_missing(component)
+                return append_meta(component.ref, meta)
+            else:
+                return append_meta(schema, meta)
 
         # nested serializer with many=True gets automatically replaced with ListSerializer
         if is_list_serializer(field):
@@ -480,7 +487,7 @@ class AutoSchema(ViewInspector):
                 component = self.resolve_serializer(field.child, direction)
                 return append_meta(build_array_type(component.ref), meta) if component else None
             else:
-                schema = self._map_serializer_field(field.child, direction, collect_meta)
+                schema = self._map_serializer_field(field.child, direction)
                 return append_meta(build_array_type(schema), meta)
 
         # nested serializer
@@ -490,7 +497,7 @@ class AutoSchema(ViewInspector):
 
         # Related fields.
         if isinstance(field, serializers.ManyRelatedField):
-            schema = self._map_serializer_field(field.child_relation, direction, collect_meta)
+            schema = self._map_serializer_field(field.child_relation, direction)
             # remove hand-over initkwargs applying only to outer scope
             schema.pop('description', None)
             schema.pop('readOnly', None)
@@ -548,7 +555,7 @@ class AutoSchema(ViewInspector):
                 component = self.resolve_serializer(field.child, direction)
                 return append_meta(build_array_type(component.ref), meta) if component else None
             else:
-                schema = self._map_serializer_field(field.child, direction, collect_meta)
+                schema = self._map_serializer_field(field.child, direction)
                 # remove automatically attached but redundant title
                 if is_trivial_string_variation(field.field_name, schema.get('title')):
                     schema.pop('title', None)
@@ -638,9 +645,7 @@ class AutoSchema(ViewInspector):
         if isinstance(field, (serializers.DictField, serializers.HStoreField)):
             content = build_basic_type(OpenApiTypes.OBJECT)
             if not isinstance(field.child, _UnvalidatedField):
-                content['additionalProperties'] = self._map_serializer_field(
-                    field.child, direction, collect_meta
-                )
+                content['additionalProperties'] = self._map_serializer_field(field.child, direction)
             return append_meta(content, meta)
 
         if isinstance(field, serializers.CharField):
@@ -649,8 +654,15 @@ class AutoSchema(ViewInspector):
         if isinstance(field, serializers.ReadOnlyField):
             # direct source from the serializer
             assert field.source_attrs, f'ReadOnlyField "{field}" needs a proper source'
-            target = follow_field_source(field.parent.Meta.model, field.source_attrs)
+            # when field is nested inside a ListSerializer, the Meta class is 2 steps removed
+            if is_list_serializer(field.parent):
+                model = field.parent.parent.Meta.model
+                source = field.parent.source_attrs
+            else:
+                model = field.parent.Meta.model
+                source = field.source_attrs
 
+            target = follow_field_source(model, source)
             if callable(target):
                 schema = self._map_response_type_hint(target)
             elif isinstance(target, models.Field):
@@ -714,11 +726,6 @@ class AutoSchema(ViewInspector):
 
     def _get_serializer_field_meta(self, field):
         if not isinstance(field, serializers.Field):
-            warn(
-                f'unable to extract field metadata from field "{field}" because it appears '
-                f'to be neither a Field nor a Serializer. Proper handling may require an '
-                f'Extension or @extend_schema_field. Skipping metadata extraction. '
-            )
             return {}
 
         meta = {}
@@ -1179,16 +1186,19 @@ class AutoSchema(ViewInspector):
 
             self.registry.register(component)
             component.schema = self._map_serializer(serializer, direction)
-            # 4 cases:
-            #   1. polymorphic container component -> use
-            #   2. concrete component with properties -> use
-            #   3. concrete component without properties -> prob. transactional so discard
-            #   4. explicit list component -> demultiplexed at usage location so discard
-            keep_component = (
-                any(nest_tag in component.schema for nest_tag in ['oneOf', 'allOf', 'anyOf'])
-                or component.schema.get('properties', {})
+
+            discard_component = (
+                # components with empty schemas serve no purpose
+                not component.schema
+                # concrete component without properties are likely only transactional so discard
+                or (
+                    component.schema.get('type') == 'object'
+                    and not component.schema.get('properties')
+                    and 'additionalProperties' not in component.schema
+                )
             )
-            if not keep_component:
+
+            if discard_component:
                 del self.registry[component]
                 return ResolvedComponent(None, None)  # sentinel
             return component
