@@ -46,6 +46,8 @@ SPECTACULAR_DEFAULTS: Dict[str, Any] = {
     'SWAGGER_UI_DIST': '//unpkg.com/swagger-ui-dist@3.44.0',
     'SWAGGER_UI_FAVICON_HREF': '//unpkg.com/swagger-ui-dist@3.44.0/favicon-32x32.png',
 
+    'REDOC_DIST': '//cdn.jsdelivr.net/npm/redoc@next',
+
     # Append OpenAPI objects to path and components in addition to the generated objects
     'APPEND_PATHS': {},
     'APPEND_COMPONENTS': {},
@@ -68,6 +70,12 @@ SPECTACULAR_DEFAULTS: Dict[str, Any] = {
     # Example: 'drf_spectacular.hooks.preprocess_exclude_path_format'
     'PREPROCESSING_HOOKS': [],
 
+    # Determines how operations should be sorted. If you intend to do sorting with a
+    # PREPROCESSING_HOOKS, be sure to disable this setting. If configured, the sorting
+    # is applied after the PREPROCESSING_HOOKS. Accepts either
+    # True (drf-spectacular's alpha-sorter), False, or a callable for sort's key arg.
+    'SORT_OPERATIONS': True,
+
     # enum name overrides. dict with keys "YourEnum" and their choice values "field.choices"
     'ENUM_NAME_OVERRIDES': {},
     # Adds "blank" and "null" enum choices where appropriate. disable on client generation issues
@@ -139,6 +147,7 @@ IMPORT_STRINGS = [
     'PREPROCESSING_HOOKS',
     'GET_LIB_DOC_EXCLUDES',
     'GET_MOCK_REQUEST',
+    'SORT_OPERATIONS',
     'SORT_OPERATION_PARAMETERS',
 ]
 
