@@ -10,10 +10,10 @@ from drf_spectacular.drainage import error, get_view_methods, set_override, warn
 from drf_spectacular.types import OpenApiTypes
 
 _SerializerType = Union[Serializer, Type[Serializer]]
-_SerializerTypeVariations = Union[OpenApiTypes, 'PolymorphicProxySerializer', _SerializerType]
+_SerializerTypeVariations = Union[OpenApiTypes, _SerializerType]
 
 
-class PolymorphicProxySerializer:
+class PolymorphicProxySerializer(Serializer):
     """
     This class is to be used with :func:`@extend_schema <.extend_schema>` to
     signal a request/response might be polymorphic (accepts/returns data
@@ -33,16 +33,17 @@ class PolymorphicProxySerializer:
         def create(self, request, *args, **kwargs):
             return Response(...)
 
-    Beware that this is not a real serializer and therefore is not derived from
-    serializers.Serializer. It *cannot* be used in views as `serializer_class`
-    or as field in a actual serializer. You likely want to handle this in the
-    view method.
+    **Beware** that this is not a real serializer and it will raise an AssertionError
+    if used in that way. It **cannot** be used in views as `serializer_class`
+    or as field in an actual serializer. It is solely meant for annotation purposes.
 
     Also make sure that each sub-serializer has a field named after the value of
     ``resource_type_field_name`` (discriminator field). Generated clients will likely
-    depend on the existence of this field.
+    depend on the existence of this field. Setting ``resource_type_field_name``
+    to ``None`` will remove the discriminator altogether. This may be useful in
+    certain situations, but will most likely break client generation.
 
-    For that reason, it is **strongly** recommended to pass the ``Serializers`` as **list**,
+    It is **strongly** recommended to pass the ``Serializers`` as **list**,
     and by that let *drf-spectacular* retrieve the field and handle the mapping
     automatically. In special circumstances, the field may not available when
     drf-spectacular processes the serializer. In those cases you can explicitly state
@@ -53,13 +54,29 @@ class PolymorphicProxySerializer:
             self,
             component_name: str,
             serializers: Union[List[_SerializerType], Dict[str, _SerializerType]],
-            resource_type_field_name: str
+            resource_type_field_name: Optional[str],
+            many: bool = False,
     ):
         self.component_name = component_name
         self.serializers = serializers
         self.resource_type_field_name = resource_type_field_name
-        self.partial = False
-        self.read_only = False
+        super().__init__(many=many)
+
+    @property
+    def data(self):
+        self._trap()
+
+    def to_internal_value(self, data):
+        self._trap()
+
+    def to_representation(self, instance):
+        self._trap()
+
+    def _trap(self):
+        raise AssertionError(
+            "PolymorphicProxySerializer is an annotation helper and not supposed to "
+            "be used for real requests. See documentation for correct usage."
+        )
 
 
 class OpenApiSchemaBase:
@@ -68,12 +85,13 @@ class OpenApiSchemaBase:
 
 class OpenApiExample(OpenApiSchemaBase):
     """
-    https://swagger.io/specification/
-     * Specification
-        * Schema
-           * ExampleObject
-    """
+    Helper class to document a API parameter / request body / response body
+    with a concrete example value.
 
+    The example will be attached to the operation object where appropriate,
+    i. e. where the given ``media_type``, ``status_code`` and modifiers match.
+    Example that do not match any scenario are ignored.
+    """
     def __init__(
             self,
             name: str,
@@ -98,6 +116,16 @@ class OpenApiExample(OpenApiSchemaBase):
 
 
 class OpenApiParameter(OpenApiSchemaBase):
+    """
+    Helper class to document request query/path/header/cookie parameters.
+    Can also be used to document response headers.
+
+    Please note that not all arguments apply to all ``location``/``type``/direction
+    variations, e.g. path parameters are ``required=True`` by definition.
+
+    For valid ``style`` choices please consult the
+    `OpenAPI specification <https://swagger.io/specification/#style-values>`_.
+    """
     QUERY = 'query'
     PATH = 'path'
     HEADER = 'header'
@@ -138,7 +166,10 @@ class OpenApiResponse(OpenApiSchemaBase):
     """
     Helper class to bundle a response object (``Serializer``, ``OpenApiType``,
     raw schema, etc) together with a response object description and/or examples.
-    Examples can alternatively be provided via @extend_schema.
+    Examples can alternatively be provided via :func:`@extend_schema <.extend_schema>`.
+
+    This class is especially helpful for explicitly describing status codes on a
+    "Response Object" level.
     """
     def __init__(
             self,
@@ -168,8 +199,8 @@ def extend_schema(
         examples: Optional[List[OpenApiExample]] = None,
 ):
     """
-    decorator mainly for the "view" method kind. partially or completely overrides
-    what would be generated by drf-spectacular.
+    Decorator mainly for the "view" method kind. Partially or completely overrides
+    what would be otherwise generated by drf-spectacular.
 
     :param operation_id: replaces the auto-generated operation_id. make sure there
         are no naming collisions.
@@ -366,7 +397,7 @@ def extend_schema_serializer(
 ):
     """
     Decorator for the "serializer" kind. Intended for overriding default serializer behaviour that
-    cannot be influenced through `.extend_schema`.
+    cannot be influenced through :func:`@extend_schema <.extend_schema>`.
 
     :param many: override how serializer is initialized. Mainly used to coerce the list view detection
         heuristic to acknowledge a non-list serializer.
@@ -391,9 +422,13 @@ def extend_schema_view(**kwargs):
     Convenience decorator for the "view" kind. Intended for annotating derived view methods that
     are are not directly present in the view (usually methods like ``list`` or ``retrieve``).
     Spares you from overriding methods like ``list``, only to perform a super call in the body
-    so that you have have something to attach ``@extend_schema`` to.
+    so that you have have something to attach :func:`@extend_schema <.extend_schema>` to.
 
-    :param kwargs: method names as argument names and ``extend_schema()`` calls as values
+    This decorator also takes care of safely attaching annotations to derived view methods,
+    preventing leakage into unrelated views.
+
+    :param kwargs: method names as argument names and :func:`@extend_schema <.extend_schema>`
+      calls as values
     """
     def wrapping_decorator(method_decorator, method):
         @method_decorator
@@ -429,8 +464,9 @@ def extend_schema_view(**kwargs):
 
 def inline_serializer(name: str, fields: Dict[str, Field], **kwargs) -> Serializer:
     """
-    A helper function to create an inline serializer. Primary use is with `@extend_schema`,
-    where one needs an implicit one-off serializer that is not reflected in an actual class.
+    A helper function to create an inline serializer. Primary use is with
+    :func:`@extend_schema <.extend_schema>`, where one needs an implicit one-off
+    serializer that is not reflected in an actual class.
 
     :param name: name of the
     :param fields: dict with field names as keys and serializer fields as values
