@@ -14,12 +14,16 @@ def pytest_configure(config):
     )
 
     contrib_apps = [
-        'rest_framework_jwt',
+        'dj_rest_auth',
+        'dj_rest_auth.registration',
+        'allauth',
+        'allauth.account',
         'oauth2_provider',
         'django_filters',
         # this is not strictly required and when added django-polymorphic
         # currently breaks the whole Django/DRF upstream testing.
         # 'polymorphic',
+        # 'rest_framework_jwt',
     ]
 
     base_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
