@@ -1,4 +1,6 @@
+import collections
 import json
+import re
 import sys
 import typing
 from datetime import datetime
@@ -11,7 +13,8 @@ from rest_framework import generics, serializers
 
 from drf_spectacular.openapi import AutoSchema
 from drf_spectacular.plumbing import (
-    detype_pattern, follow_field_source, force_instance, is_field, is_serializer, resolve_type_hint,
+    analyze_named_regex_pattern, detype_pattern, follow_field_source, force_instance, is_field,
+    is_serializer, resolve_type_hint,
 )
 from drf_spectacular.validation import validate_schema
 from tests import generate_schema
@@ -81,6 +84,11 @@ def test_detype_patterns_with_module_includes(no_warnings):
     )
 
 
+class NamedTupleB(typing.NamedTuple):
+    a: int
+    b: str
+
+
 TYPE_HINT_TEST_PARAMS = [
     (
         typing.Optional[int],
@@ -93,7 +101,7 @@ TYPE_HINT_TEST_PARAMS = [
         {'type': 'array', 'items': {'type': 'object', 'additionalProperties': {'type': 'integer'}}}
     ), (
         list,
-        {'type': 'array', 'items': {'type': 'object', 'additionalProperties': {}}}
+        {'type': 'array', 'items': {}}
     ), (
         typing.Tuple[int, int, int],
         {'type': 'array', 'items': {'type': 'integer'}, 'minLength': 3, 'maxLength': 3}
@@ -122,11 +130,28 @@ TYPE_HINT_TEST_PARAMS = [
         typing.Union[int, str, None],
         {'oneOf': [{'type': 'integer'}, {'type': 'string'}], 'nullable': True}
     ), (
-        typing.Optional[typing.Union[int, str]],
-        {'oneOf': [{'type': 'integer'}, {'type': 'string'}], 'nullable': True}
+        typing.Optional[typing.Union[str, int]],
+        {'oneOf': [{'type': 'string'}, {'type': 'integer'}], 'nullable': True}
     )
 ]
 
+if sys.version_info >= (3, 7):
+    TYPE_HINT_TEST_PARAMS.append((
+        typing.Iterable[collections.namedtuple("NamedTupleA", "a, b")],  # noqa
+        {
+            'type': 'array',
+            'items': {'type': 'object', 'properties': {'a': {}, 'b': {}}, 'required': ['a', 'b']}
+        }
+    ))
+    TYPE_HINT_TEST_PARAMS.append((
+        NamedTupleB,
+        {
+            'type': 'object',
+            'properties': {'a': {'type': 'integer'}, 'b': {'type': 'string'}},
+            'required': ['a', 'b']
+        }
+    ))
+
 if sys.version_info >= (3, 8):
     TYPE_HINT_TEST_PARAMS.append((
         typing.Literal['x', 'y'],
@@ -192,3 +217,17 @@ def test_type_hint_extraction(no_warnings, type_hint, ref_schema):
         serializer_class = XSerializer
 
     validate_schema(generate_schema('/x', view=XView))
+
+
+@pytest.mark.parametrize(['pattern', 'output'], [
+    ('(?P<t1><,()(())(),)', {'t1': '<,()(())(),'}),
+    (r'(?P<t1>.\\)', {'t1': r'.\\'}),
+    (r'(?P<t1>.\\\\)', {'t1': r'.\\\\'}),
+    (r'(?P<t1>.\))', {'t1': r'.\)'}),
+    (r'(?P<t1>)', {'t1': r''}),
+    (r'(?P<t1>.[\(]{2})', {'t1': r'.[\(]{2}'}),
+    (r'(?P<t1>(.))/\(t/(?P<t2>\){2}()\({2}().*)', {'t1': '(.)', 't2': r'\){2}()\({2}().*'}),
+])
+def test_analyze_named_regex_pattern(no_warnings, pattern, output):
+    re.compile(pattern)  # check validity of regex
+    assert analyze_named_regex_pattern(pattern) == output
