@@ -1,3 +1,4 @@
+import datetime
 import typing
 import uuid
 from decimal import Decimal
@@ -17,16 +18,14 @@ from rest_framework.decorators import action, api_view
 from rest_framework.views import APIView
 
 from drf_spectacular.extensions import OpenApiSerializerExtension
-from drf_spectacular.generators import SchemaGenerator
 from drf_spectacular.hooks import preprocess_exclude_path_format
 from drf_spectacular.openapi import AutoSchema
-from drf_spectacular.renderers import OpenApiYamlRenderer
+from drf_spectacular.renderers import OpenApiJsonRenderer, OpenApiYamlRenderer
 from drf_spectacular.types import OpenApiTypes
 from drf_spectacular.utils import (
     OpenApiExample, OpenApiParameter, OpenApiResponse, extend_schema, extend_schema_field,
     extend_schema_serializer, extend_schema_view, inline_serializer,
 )
-from drf_spectacular.validation import validate_schema
 from tests import generate_schema, get_request_schema, get_response_schema
 from tests.models import SimpleModel, SimpleSerializer
 
@@ -233,12 +232,10 @@ def test_free_form_responses(no_warnings):
         def get(self, request):
             pass  # pragma: no cover
 
-    generator = SchemaGenerator(patterns=[
+    generate_schema(None, patterns=[
         re_path(r'^x$', XAPIView.as_view(), name='x'),
         re_path(r'^y$', YAPIView.as_view(), name='y'),
     ])
-    schema = generator.get_schema(request=None, public=True)
-    validate_schema(schema)
 
 
 @mock.patch(
@@ -254,12 +251,10 @@ def test_append_extra_components(no_warnings):
         def get(self, request):
             pass  # pragma: no cover
 
-    generator = SchemaGenerator(patterns=[
+    schema = generate_schema(None, patterns=[
         re_path(r'^x$', XAPIView.as_view(), name='x'),
     ])
-    schema = generator.get_schema(request=None, public=True)
     assert len(schema['components']['schemas']) == 2
-    validate_schema(schema)
 
 
 def test_serializer_retrieval_from_view(no_warnings):
@@ -285,9 +280,7 @@ def test_serializer_retrieval_from_view(no_warnings):
     router = routers.SimpleRouter()
     router.register('x1', X1Viewset, basename='x1')
     router.register('x2', X2Viewset, basename='x2')
-    generator = SchemaGenerator(patterns=router.urls)
-    schema = generator.get_schema(request=None, public=True)
-    validate_schema(schema)
+    schema = generate_schema(None, patterns=router.urls)
     assert len(schema['components']['schemas']) == 2
     assert 'Unused' not in schema['components']['schemas']
 
@@ -493,9 +486,7 @@ def test_drf_format_suffix_parameter(no_warnings, allowed):
     ]
     urlpatterns = format_suffix_patterns(urlpatterns, allowed=allowed)
 
-    generator = SchemaGenerator(patterns=urlpatterns)
-    schema = generator.get_schema(request=None, public=True)
-    validate_schema(schema)
+    schema = generate_schema(None, patterns=urlpatterns)
 
     # Only seven alternatives are created, as /pi/{format} would be
     # /pi/.json which is not supported.
@@ -535,9 +526,7 @@ def test_drf_format_suffix_parameter_exclude(no_warnings):
     urlpatterns = format_suffix_patterns([
         path('pi', view_func),
     ])
-    generator = SchemaGenerator(patterns=urlpatterns)
-    schema = generator.get_schema(request=None, public=True)
-    validate_schema(schema)
+    schema = generate_schema(None, patterns=urlpatterns)
     assert list(schema['paths'].keys()) == ['/pi']
 
 
@@ -548,9 +537,7 @@ def test_regex_path_parameter_discovery(no_warnings):
         pass  # pragma: no cover
 
     urlpatterns = [re_path(r'^/pi/<int:precision>', pi)]
-    generator = SchemaGenerator(patterns=urlpatterns)
-    schema = generator.get_schema(request=None, public=True)
-    validate_schema(schema)
+    schema = generate_schema(None, patterns=urlpatterns)
     parameter = schema['paths']['/pi/{precision}']['get']['parameters'][0]
     assert parameter['name'] == 'precision'
     assert parameter['in'] == 'path'
@@ -877,7 +864,7 @@ def test_extension_subclass_discovery(no_warnings):
         pass
 
     class XSerializer(serializers.Serializer):
-        field = serializers.IntegerField
+        field = serializers.IntegerField()
 
     class XAPIView(APIView):
         authentication_classes = [CustomAuth]
@@ -955,9 +942,7 @@ def test_schema_contains_only_urlpatterns_first_match(no_warnings):
         path('api/x/', XAPIView.as_view()),  # only first occurrence is used
         path('api/x/', YAPIView.as_view()),
     ]
-    generator = SchemaGenerator(patterns=urlpatterns)
-    schema = generator.get_schema(request=None, public=True)
-    validate_schema(schema)
+    schema = generate_schema(None, patterns=urlpatterns)
     assert len(schema['components']['schemas']) == 1
     assert 'X' in schema['components']['schemas']
     operation = schema['paths']['/api/x/']['get']
@@ -1206,9 +1191,7 @@ def test_pagination_reusage(no_warnings):
     router = routers.SimpleRouter()
     router.register('x', XViewset, basename='x')
     router.register('y', YViewset, basename='y')
-    generator = SchemaGenerator(patterns=router.urls)
-    schema = generator.get_schema(request=None, public=True)
-    validate_schema(schema)
+    generate_schema(None, patterns=router.urls)
 
 
 def test_pagination_disabled_on_action(no_warnings):
@@ -1255,7 +1238,7 @@ def test_manual_security_method_addition(no_warnings):
 
 def test_basic_viewset_without_queryset_with_explicit_pk_typing(no_warnings):
     class XSerializer(serializers.Serializer):
-        field = fields.IntegerField()
+        field = serializers.IntegerField()
 
     class XViewset(viewsets.ViewSet):
         serializer_class = XSerializer
@@ -1266,9 +1249,7 @@ def test_basic_viewset_without_queryset_with_explicit_pk_typing(no_warnings):
     urlpatterns = [
         path("api/<path:some_var>/<uuid:pk>/", XViewset.as_view({"get": "retrieve"}))
     ]
-    generator = SchemaGenerator(patterns=urlpatterns)
-    schema = generator.get_schema(request=None, public=True)
-    validate_schema(schema)
+    schema = generate_schema(None, patterns=urlpatterns)
     operation = schema['paths']['/api/{some_var}/{id}/']['get']
     assert operation['parameters'][0]['name'] == 'id'
     assert operation['parameters'][0]['schema']['format'] == 'uuid'
@@ -1472,7 +1453,7 @@ def test_path_param_from_related_model_pk_without_primary_key_true(no_warnings,
     router = routers.SimpleRouter()
     router.register(path, XViewset)
 
-    schema = SchemaGenerator(patterns=router.urls).get_schema(request=None, public=True)
+    schema = generate_schema(None, patterns=router.urls)
     assert '/x/{related_field}/' in schema['paths']
     assert '/x/{related_field}/{id}/' in schema['paths']
 
@@ -1581,6 +1562,29 @@ def test_parameter_sorting_setting(no_warnings, sorting, result):
         assert [p['name'] for p in parameters] == result
 
 
+@pytest.mark.parametrize(['sorting', 'result'], [
+    (True, ['/a/', '/b/', '/c/']),
+    (False, ['/c/', '/a/', '/b/']),
+    (lambda x: {'/c/': 1, '/b/': 2, '/a/': 3}.get(x[0]), ['/c/', '/b/', '/a/']),
+])
+def test_operation_sorting_setting(no_warnings, sorting, result):
+    @extend_schema(responses=OpenApiTypes.ANY)
+    @api_view(['GET'])
+    def view_func(request, format=None):
+        pass  # pragma: no cover
+
+    urlpatterns = [
+        path('/c/', view_func),
+        path('/a/', view_func),
+        path('/b/', view_func),
+    ]
+    with mock.patch(
+        'drf_spectacular.settings.spectacular_settings.SORT_OPERATIONS', sorting
+    ):
+        schema = generate_schema(None, patterns=urlpatterns)
+        assert list(schema['paths'].keys()) == result
+
+
 def test_response_headers_without_response_body(no_warnings):
     @extend_schema(
         responses={301: None},
@@ -1655,7 +1659,7 @@ def test_any_placeholder_on_request_response():
         custom_field = CustomField()
 
         def get_method_field(self, obj) -> typing.Any:
-            return
+            return  # pragma: no cover
 
     @extend_schema(request=typing.Any, responses=XSerializer)
     @api_view(['POST'])
@@ -1788,6 +1792,7 @@ def test_openapi_response_wrapper(no_warnings):
             'description': 'a list that actually returns numbers'
         }
     }
+    assert schema['paths']['/x/']['post']['description'] == 'creation description'
     assert schema['paths']['/x/']['post']['responses'] == {
         '201': {
             'content': {'application/json': {'schema': {'type': 'integer'}}},
@@ -1811,9 +1816,179 @@ def test_prefix_estimation_with_re_special_chars_as_literals_in_path(no_warnings
     def view_func2(request, format=None):
         pass  # pragma: no cover
 
-    generator = SchemaGenerator(patterns=[
+    schema = generate_schema(None, patterns=[
         path('/\\/x/', view_func1),
         path('/\\/y/', view_func2)
     ])
-    schema = generator.get_schema(request=None, public=True)
     assert schema['paths']['/\\/x/']['post']['tags'] == ['x']
+
+
+def test_nested_router_urls(no_warnings):
+    # somewhat tailored to drf-nested-routers but also serves a generic purpose
+    # as "id" coercion also makes sense for "_pk" suffixes.
+    class RouteNestedMaildropModel(models.Model):
+        renamed_id = models.IntegerField(primary_key=True)
+
+    class RouteNestedClientModel(models.Model):
+        id = models.UUIDField(primary_key=True)
+
+    class RouteNestedModel(models.Model):
+        client = models.ForeignKey(RouteNestedClientModel, on_delete=models.CASCADE)
+        maildrop = models.ForeignKey(RouteNestedMaildropModel, on_delete=models.CASCADE)
+
+    class RouteNestedViewset(viewsets.ModelViewSet):
+        queryset = RouteNestedModel.objects.all()
+        serializer_class = SimpleSerializer
+
+    urlpatterns = [
+        path(
+            '/clients/{client_pk}/maildrops/{maildrop_pk}/recipients/{pk}/',
+            RouteNestedViewset.as_view({'get': 'retrieve'})
+        ),
+    ]
+    schema = generate_schema(None, patterns=urlpatterns)
+    operation = schema['paths']['/clients/{client_id}/maildrops/{maildrop_id}/recipients/{id}/']['get']
+    assert operation['parameters'][0]['name'] == 'client_id'
+    assert operation['parameters'][0]['schema'] == {'format': 'uuid', 'type': 'string'}
+    assert operation['parameters'][2]['name'] == 'maildrop_id'
+    assert operation['parameters'][2]['schema'] == {'type': 'integer'}
+
+
+@pytest.mark.parametrize('value', [
+    datetime.datetime(year=2021, month=1, day=1),
+    datetime.date(year=2021, month=1, day=1),
+    datetime.time(),
+    datetime.timedelta(days=1),
+    uuid.uuid4(),
+    Decimal(),
+    b'deadbeef'
+])
+def test_yaml_encoder_parity(no_warnings, value):
+    # make sure our YAML renderer does not choke on objects that are fine with
+    # rest_framework.encoders.JSONEncoder
+    assert OpenApiJsonRenderer().render(value)
+    assert OpenApiYamlRenderer().render(value)
+
+
+@pytest.mark.parametrize(['comp_schema', 'discarded'], [
+    ({'type': 'object'}, True),
+    ({'type': 'object', 'properties': {}}, True),
+    ({'type': 'object', 'additionalProperties': {}}, False),
+    ({'type': 'object', 'additionalProperties': {'type': 'number'}}, False),
+    ({'type': 'number'}, False),
+    ({'type': 'array', 'items': {'type': 'number'}}, False),
+])
+def test_serializer_extension_with_non_object_schema(no_warnings, comp_schema, discarded):
+    class XSerializer(serializers.Serializer):
+        field = serializers.CharField()
+
+    class XExtension(OpenApiSerializerExtension):
+        target_class = XSerializer
+
+        def map_serializer(self, auto_schema, direction):
+            return comp_schema
+
+    class XAPIView(APIView):
+        @extend_schema(request=XSerializer, responses=XSerializer)
+        def post(self, request):
+            pass  # pragma: no cover
+
+    schema = generate_schema('x', view=XAPIView)
+
+    operation = schema['paths']['/x']['post']
+    if discarded:
+        assert 'requestBody' not in operation
+    else:
+        assert get_request_schema(operation)['$ref'] == '#/components/schemas/X'
+        assert schema['components']['schemas']['X'] == comp_schema
+
+
+def test_response_header_with_serializer_component(no_warnings):
+    class XSerializer(serializers.Serializer):
+        field = serializers.CharField()
+
+    @extend_schema(
+        request=OpenApiTypes.ANY,
+        responses=OpenApiTypes.ANY,
+        parameters=[OpenApiParameter(
+            name='test',
+            type=XSerializer,
+            location=OpenApiParameter.HEADER,
+            response=True,
+        )]
+    )
+    @api_view(['POST'])
+    def view_func(request, format=None):
+        pass  # pragma: no cover
+
+    schema = generate_schema('x', view_function=view_func)
+    assert 'X' in schema['components']['schemas']
+    assert schema['paths']['/x']['post']['responses']['200']['headers'] == {
+        'test': {'schema': {'$ref': '#/components/schemas/X'}}
+    }
+
+
+def test_extend_schema_noop_request_content_type(no_warnings):
+    @extend_schema(
+        request={
+            'application/json': None,  # for completeness, not necessary
+            'application/pdf': OpenApiTypes.BINARY
+        },
+        responses=OpenApiTypes.ANY,
+    )
+    @api_view(['POST'])
+    def view_func(request, format=None):
+        pass  # pragma: no cover
+
+    schema = generate_schema('x', view_function=view_func)
+    assert 'application/pdf' in schema['paths']['/x']['post']['requestBody']['content']
+    assert 'application/json' not in schema['paths']['/x']['post']['requestBody']['content']
+
+
+def test_viewset_reverse_list_detection_override(no_warnings):
+    class XViewset(viewsets.ReadOnlyModelViewSet):
+        queryset = SimpleModel.objects.all()
+        serializer_class = SimpleSerializer
+
+        @extend_schema(
+            # without explicit operation_id, this operation is detected as non-list and thus
+            # will be named "x_retrieve", which create a collision with the actual retrieve.
+            operation_id='x_list',
+            parameters=[OpenApiParameter("format")],
+            responses={(200, "*/*"): OpenApiTypes.STR},
+        )
+        def list(self, request, *args, **kwargs):
+            pass  # pragma: no cover
+
+    schema = generate_schema('/x', XViewset)
+    assert schema['paths']['/x/']['get']['parameters'][0]['name'] == 'format'
+    assert schema['paths']['/x/']['get']['operationId'] == 'x_list'
+    assert schema['paths']['/x/{id}/']['get']['operationId'] == 'x_retrieve'
+
+
+def test_list_serializer_with_read_only_field_on_model_property(no_warnings):
+    class M7Model(models.Model):
+        @property
+        def all_groups(self) -> typing.List[int]:
+            return [1, 2, 3]
+
+    class XField(serializers.ReadOnlyField):
+        pass
+
+    class XSerializer(serializers.ModelSerializer):
+        groups = serializers.ListSerializer(source="all_groups", child=XField(), read_only=True)
+
+        class Meta:
+            model = M7Model
+            fields = '__all__'
+
+    class XViewset(viewsets.ReadOnlyModelViewSet):
+        queryset = M7Model.objects.none()
+        serializer_class = XSerializer
+
+    schema = generate_schema('x', XViewset)
+    assert schema['components']['schemas']['X']['properties']['groups'] == {
+        'type': 'array',
+        'items': {'type': 'array', 'items': {'type': 'integer'}, 'readOnly': True},
+        'readOnly': True
+    }
